{% extends 'generic_scaffold/base.html' %}
{% block content %}
{{ object }}
<form method='POST'>
    {% csrf_token %}
    <input type='submit'>
</form>
<a href='{% url object.list_url_name %}'>List</a>
{% endblock %}