{% extends 'generic_scaffold/base.html' %}
{% load generic_scaffold_tags %}
{% block content %}

<a href='{% url object_list.model.create_url_name %}'>Create</a>
<ul>
    {% for object in object_list %}
        <li>
            {{ object }}
            <a href='{% url object.detail_url_name object.id %}'>detail</a>
            <a href='{% url object.update_url_name object.id %}'>update</a>
            <a href='{% url object.delete_url_name object.id %}'>delete</a>
        </li>
    {% endfor %}
</ul>
{% endblock %}